#include "MainWindow.h"

using namespace cv;

MainWindow::MainWindow(){
	QMdiArea *m_centralZone = new QMdiArea;

	/*******CREATION OF THE MAIN WINDOW INTERFACE******/

	setGeometry(40,40,1000,600); //Initialisation of the main window size

	//Creation of the menu
	QMenu *fileMenu = menuBar()->addMenu("File");
		QAction *openAction = fileMenu->addAction("Open HDF");
		openAction->setShortcut(QKeySequence("Ctrl+O"));
		openAction->setIcon(QPixmap("C:/Users/Nico/Documents/Visual Studio 2012/Projects/ImageProcessing/Win32/Debug/open.png"));
		QAction *openImageAction = fileMenu->addAction("Open Image");
		m_testImageAction = fileMenu->addAction("Test");
		m_testImageAction->setCheckable(true);


	QMenu *toolMenu = menuBar()->addMenu("Tools");
		m_zoomInAction = toolMenu->addAction("Zoom In");
		m_zoomInAction->setCheckable(true);
		m_zoomInAction->setShortcut(QKeySequence("Ctrl+I"));
		m_zoomInAction->setIcon(QPixmap("C:/Users/Nico/Documents/Visual Studio 2012/Projects/ImageProcessing/Win32/Debug/zoomin.png"));
		m_zoomOutAction = toolMenu->addAction("Zoom Out");
		m_zoomOutAction->setCheckable(true);
		m_zoomOutAction->setShortcut(QKeySequence("Ctrl+U"));
		m_zoomOutAction->setIcon(QPixmap("C:/Users/Nico/Documents/Visual Studio 2012/Projects/ImageProcessing/Win32/Debug/zoomout.png"));
		m_zoomZoneAction = toolMenu->addAction("Zoom Zone");
		m_zoomZoneAction->setCheckable(true);
		m_zoomZoneAction->setShortcut(QKeySequence("Ctrl+R"));
		m_zoomZoneAction->setIcon(QPixmap("C:/Users/Nico/Documents/Visual Studio 2012/Projects/ImageProcessing/Win32/Debug/zoomzone.png"));
	

	QObject::connect(openAction,SIGNAL(triggered()),this,SLOT(openActionSlot()));
	QObject::connect(openImageAction,SIGNAL(triggered()),this,SLOT(openActionImageSlot()));
	QObject::connect(m_zoomInAction,SIGNAL(toggled(bool)),this,SLOT(changeZoomIn(bool)));
	QObject::connect(m_zoomOutAction,SIGNAL(toggled(bool)),this,SLOT(changeZoomOut(bool)));
	QObject::connect(m_zoomZoneAction,SIGNAL(toggled(bool)),this,SLOT(changeZoomZone(bool)));
	QObject::connect(m_testImageAction,SIGNAL(triggered()),this,SLOT(testImageSlot()));





	//Explorer Window
	explorerWindow = new ExplorerWindow(tr("Explorer"),this);
	addDockWidget(Qt::LeftDockWidgetArea, explorerWindow);
	explorerWindow->setFeatures(QDockWidget::NoDockWidgetFeatures);
	QObject::connect(explorerWindow,SIGNAL(SDSSelectionnedRGB(uint16**,int32*,int,QString,QString,int)),this,SLOT(saveSDSRGB(uint16**,int32*,int,QString,QString,int)));
	QObject::connect(explorerWindow,SIGNAL(itemSelectionned(QStringList,QVector<QVector<int32>>,QString,int32,QString)),this,SLOT(openVData(QStringList,QVector<QVector<int32>>,QString,int32,QString)));
	QObject::connect(explorerWindow,SIGNAL(SDSSelectionned(uint16**,int32*,int,QString,QString)),this,SLOT(openSDS(uint16**,int32*,int,QString,QString)));
	QObject::connect(explorerWindow,SIGNAL(closeFile(int)),this,SLOT(closeFileSlot(int)));

	//Tool Widgets
	toolsDock = new ToolsDock("Tools");
	toolsDock->setMinimumWidth(245);
	addDockWidget(Qt::RightDockWidgetArea, toolsDock);
	toolsDock->setFeatures(QDockWidget::NoDockWidgetFeatures);
	QObject::connect(toolsDock,SIGNAL(contrastLuminosityModified(int,int)),this,SLOT(contrastLuminosityModifiedSlot(int,int)));
	QObject::connect(toolsDock,SIGNAL(createRGBImage(QString,QString,QString)),this,SLOT(createRGBImageSlot(QString,QString,QString)));
	QObject::connect(toolsDock,SIGNAL(selectionPaternButton(bool)),this,SLOT(selectionPaternZone(bool)));
	QObject::connect(toolsDock,SIGNAL(selectionZoneButton(bool)),this,SLOT(selectionSearchingZone(bool)));
	QObject::connect(toolsDock,SIGNAL(allImageZoneSignal()),this,SLOT(allImageZone()));
	QObject::connect(toolsDock,SIGNAL(resetGMRFSignal()),this,SLOT(resetMat()));
	QObject::connect(toolsDock,SIGNAL(lbpResetSignal()),this,SLOT(resetMat()));
	QObject::connect(toolsDock,SIGNAL(neighborsChangeSignal()),this,SLOT(resetMatLBP()));


	//Tools Bar
	QToolBar *toolBarFichier = addToolBar("File");
	toolBarFichier->setMaximumHeight(25);
    toolBarFichier->addAction(openAction);
	toolBarFichier->addAction(m_zoomInAction);
	toolBarFichier->addAction(m_zoomOutAction);
	toolBarFichier->addAction(m_zoomZoneAction);

	
	//Main Window : tabWidget
	m_workZoneTab = new QTabWidget();
	m_workZoneTab->setTabsClosable(true);
	m_workZoneTab->setGeometry(0,0,1040-explorerWindow->width(),578);
	m_mainSubWindow = m_centralZone->addSubWindow(m_workZoneTab);
	m_mainSubWindow->setWindowFlags(Qt::FramelessWindowHint);
	m_mainSubWindow->setGeometry(0,0,1040-explorerWindow->width(),578);
	setCentralWidget(m_centralZone);
	QObject::connect(m_workZoneTab,SIGNAL(tabCloseRequested(int)),this,SLOT(closeTab(int)));
	QObject::connect(m_workZoneTab,SIGNAL(currentChanged(int)),this,SLOT(updateSliders(int)));

	//Resizing connexions
	QObject::connect(explorerWindow,SIGNAL(resizeExplorer(int)),this,SLOT(resizeExplorerSlot(int)));
	QObject::connect(toolsDock,SIGNAL(resizeTools(int)),this,SLOT(resizeToolsSlot(int)));

	
}
void MainWindow::closeFileSlot(int itemIndex){
	for (int i(m_openedFiles.size()-1);i>=0;i--){
		if (m_openedFiles[i]->getFileName()==m_fileNames[itemIndex]){		
			closeTab(i);
		}
	}	
	m_fileNames.remove(itemIndex);
}
void MainWindow::closeTab(int index){
	m_openedFiles.remove(index);
	m_workZoneTab->removeTab(index);

}
void MainWindow::resizeEvent(QResizeEvent *event){
	QMainWindow::resizeEvent(event);
	m_workZoneTab->setGeometry(m_workZoneTab->pos().x(),m_workZoneTab->pos().y(),width()-explorerWindow->width()-6-toolsDock->width(),height()-20-statusBar()->height()-25);
	m_mainSubWindow->setGeometry(m_mainSubWindow->pos().x(),m_mainSubWindow->pos().y(),width()-explorerWindow->width()-6-toolsDock->width(),height()-20-statusBar()->height()-25);

}
void MainWindow::resizeExplorerSlot(int widthExplorer){
	m_workZoneTab->setGeometry(m_workZoneTab->pos().x(),m_workZoneTab->pos().y(),width()-explorerWindow->width()-6-toolsDock->width(),height()-20-statusBar()->height()-25);
	m_mainSubWindow->setGeometry(m_mainSubWindow->pos().x(),m_mainSubWindow->pos().y(),width()-explorerWindow->width()-6-toolsDock->width(),height()-20-statusBar()->height()-25);


}
void MainWindow::resizeToolsSlot(int widthTools){
	m_workZoneTab->setGeometry(m_workZoneTab->pos().x(),m_workZoneTab->pos().y(),width()-explorerWindow->width()-6-toolsDock->width(),height()-20-statusBar()->height()-25);
	m_mainSubWindow->setGeometry(m_mainSubWindow->pos().x(),m_mainSubWindow->pos().y(),width()-explorerWindow->width()-6-toolsDock->width(),height()-20-statusBar()->height()-25);

}
void MainWindow::openActionSlot(){
	QString fileName(QFileDialog::getOpenFileName(this,"Open a picture : ","C:/Users/Nico/Downloads/", "Images (*.hdf)"));//Open the Dialog window to open a file
	
	//We verify if the fileName isn't empty or already opened
	bool alreadyOpened(false);
	for (int i(0); i<m_fileNames.size();i++){
		if (fileName==m_fileNames[i]){
			alreadyOpened = true;
		}
	}
	
	//We open the file
	if (!alreadyOpened && !fileName.isEmpty()){
		m_fileNames.push_back(fileName);
		explorerWindow->addFile(m_fileNames[m_fileNames.size()-1]);
	}

	//If it's already opened we display an error message
	else if (alreadyOpened){
		QMessageBox::warning(this, "Warning", "The file is already opened.");
	}
}
void MainWindow::openSDS(uint16 **data,int32 *dimsize,int sd_index,QString fileName,QString sdsName){
	int index = fileAlreadyOpened("SDS",sd_index,fileName);
	if(index == -1){ //If it isn't already opened we open it
		m_openedFiles.push_back(new OpenedFile(this,"SDS",fileName,sd_index,data,dimsize));
		m_workZoneTab->addTab(m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),sdsName);
		m_workZoneTab->setCurrentIndex(m_openedFiles.size()-1);

		//Zoom Options
		QObject::connect(m_zoomInAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomIn(bool)));
		QObject::connect(m_zoomOutAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomOut(bool)));
		QObject::connect(m_zoomZoneAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomZone(bool)));
		QObject::connect(m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SIGNAL(paternCoordinates(QPointF,QPointF)),this,SLOT(textureCalculSlot(QPointF,QPointF)));


		if (m_zoomInAction->isChecked()){
			emit m_zoomInAction->toggled(true);
		}
		if (m_zoomOutAction->isChecked()){
			emit m_zoomOutAction->toggled(true);
		}
		if (m_zoomZoneAction->isChecked()){
			emit m_zoomZoneAction->toggled(true);
		}
		//Contrast and Luminosity
		toolsDock->enableSliders();
		
	}
	else{
		m_workZoneTab->setCurrentIndex(index);
	}
	
}
void MainWindow::changeZoomIn(bool checked){
	if (checked){
		m_zoomOutAction->setChecked(false);
		m_zoomZoneAction->setChecked(false);
		toolsDock->desactivateSelectionZoneButton();
	}
}
void MainWindow::changeZoomOut(bool checked){
	if (checked){
		m_zoomInAction->setChecked(false);
		m_zoomZoneAction->setChecked(false);
		toolsDock->desactivateSelectionZoneButton();
	}
}
void MainWindow::changeZoomZone(bool checked){
	if (checked){
		m_zoomInAction->setChecked(false);
		m_zoomOutAction->setChecked(false);
		toolsDock->desactivateSelectionZoneButton();
	}
}
void MainWindow::contrastLuminosityModifiedSlot(int contrast,int luminosity){
	double contrastDouble(0);
	if (contrast>0){
		contrastDouble = (double)contrast*2.55/100.+1.;
	}
	else{
		contrastDouble = (double)contrast/100.+1.;
	}
	double luminosityDouble = (double)luminosity;
	int index(m_workZoneTab->currentIndex());
	if(contrast != m_openedFiles[index]->getContrast() || luminosity != m_openedFiles[index]->getLuminosity()){
		m_openedFiles[index]->modifyMat(contrast,luminosity,ProcessingFunctions::contrast(this,m_openedFiles[index]->getMat(),contrastDouble,luminosityDouble));
	}
}
void MainWindow::updateSliders(int globalIndex){
	int index = m_workZoneTab->currentIndex();
	if(index>=0){
		int dd = m_openedFiles[index]->getContrast();
		toolsDock->setValues(m_openedFiles[index]->getContrast(),m_openedFiles[index]->getLuminosity());
	}
	else{
		toolsDock->setValues(0,0);
	}

	toolsDock->desactivateSelectionButton();
}
void MainWindow::createRGBImageSlot(QString redBand,QString greenBand,QString blueBand){
	uint16** data1 = explorerWindow->findItem(redBand,0);
	uint16** data2 = explorerWindow->findItem(greenBand,1);
	uint16** data3 = explorerWindow->findItem(blueBand,2);
	
	int32* dimsize= explorerWindow->findDimSize(redBand);
	int32 dimsizeBuffer[2] = {dimsize[0],dimsize[1]};

		m_openedFiles.push_back(new OpenedFile(this,"RGB",data1,data2,data3,dimsizeBuffer));
		m_workZoneTab->addTab(m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),"RGB");
		m_workZoneTab->setCurrentIndex(m_openedFiles.size()-1);

		//Zoom Options
		QObject::connect(m_zoomInAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomIn(bool)));
		QObject::connect(m_zoomOutAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomOut(bool)));
		QObject::connect(m_zoomZoneAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomZone(bool)));


		if (m_zoomInAction->isChecked()){
			emit m_zoomInAction->toggled(true);
		}
		if (m_zoomOutAction->isChecked()){
			emit m_zoomOutAction->toggled(true);
		}
		if (m_zoomZoneAction->isChecked()){
			emit m_zoomZoneAction->toggled(true);
		}
		//Contrast and Luminosity
		toolsDock->enableSliders();
}
void MainWindow::selectionPaternZone(bool checked){
	m_zoomInAction->setChecked(false);
	m_zoomOutAction->setChecked(false);
	m_zoomZoneAction->setChecked(false);
	if (m_openedFiles.size()>0){
		m_openedFiles[m_workZoneTab->currentIndex()]->getGraphicsView()->activePaternSelection(checked);
	}

}
void MainWindow::textureCalculSlot(QPointF init, QPointF end){	
	//Neighbors
	int *neighborsAndMiddle(ProcessingFunctions::neignborhoodAndMiddle(toolsDock->getNeighBorsChoice()));
	int neighborhood(neighborsAndMiddle[0]);
	int middle(neighborsAndMiddle[1]);

	
	/***************************************************************************************/
	/*********Calcul the vector of features and display the patern in the toolDock**********/
	/***************************************************************************************/
	if (toolsDock->getCheckedPaternButton()){
		int dim1 = (int)end.x()-(int)init.x();
		int dim2 = (int)end.y()-(int)init.y();

		//Get the pattern in a mat structure and display it
		cv::Mat matPattern = Mat(dim2,dim1, CV_8UC3);
		for (int i(0);i<dim2;i++){
			for (int j(0); j<dim1;j++){
				matPattern.at<Vec3b>(i,j)[0] =m_openedFiles[m_workZoneTab->currentIndex()]->getModifiedMat().at<Vec3b>(i+(int)init.y(),j+(int)init.x())[0];
				matPattern.at<Vec3b>(i,j)[1] =m_openedFiles[m_workZoneTab->currentIndex()]->getModifiedMat().at<Vec3b>(i+(int)init.y(),j+(int)init.x())[1];
				matPattern.at<Vec3b>(i,j)[2] =m_openedFiles[m_workZoneTab->currentIndex()]->getModifiedMat().at<Vec3b>(i+(int)init.y(),j+(int)init.x())[2];
			}
		}
		const uchar *qImageBuffer = (const uchar*)matPattern.data; //conversion from Mat to uchar*
		QImage image = QImage(qImageBuffer,matPattern.cols, matPattern.rows,matPattern.step, QImage::Format_RGB888);
		toolsDock->displayPaternImage(image);

		


		/***LBP METHOD***/
		if(toolsDock->getTextureMethod() == 0) {
			QVector<double> histogramBuffer(ProcessingFunctions::LBPCalculRef(m_openedFiles[m_workZoneTab->currentIndex()]->getModifiedMat(),init,end,neighborhood,middle));
			double sum(0.);
			for (int i(0);i<(int)pow(2,16)+1;i++){
				sum +=histogramBuffer[i];
			}
			//Add the vector to the features
			m_lbpFeatures.push_back(histogramBuffer);
			

			//Add the label to the table
			toolsDock->lbpAddLabel(QString::number(ProcessingFunctions::indexMeanCalcul(histogramBuffer,neighborhood)));
			toolsDock->addHistogramPublic(histogramBuffer,toolsDock->getColorCurrentLabel(),neighborhood);

			toolsDock->desactivateSelectionButton();
			if (m_lbpFeatures.size()>1){
				toolsDock->enableSearchingZoneButton(true);
			}
		} //LBP

		/***GMRF METHOD***/
		else if (toolsDock->getTextureMethod() == 1) { 
			double **inverseMatrix = new double*[(int)pow(neighborhood,2)];
			for (int i(0);i<(int)pow(neighborhood,2);i++){
				inverseMatrix[i] = new double[(int)pow(neighborhood,2)];
			}
			double *mean = new double[(int)pow(neighborhood,2)];
			ProcessingFunctions::GMRFCalculRefMean(matPattern,neighborhood,mean);
			m_featuresMean.push_back(mean);
			//ProcessingFunctions::GMRFCalculRefMatrix(matPattern,neighborhood,m_featuresMean[m_featuresMean.size()-1],inverseMatrix);
			m_featuresMatrix.push_back(inverseMatrix);



			

			//Add to the table
			toolsDock->addLabel();
			//toolsDock->addGaussianPublic(featuresBuffer,toolsDock->getColorCurrentLabel());

			toolsDock->desactivateSelectionButton();
			if (m_featuresMean.size()>1){
				toolsDock->enableSearchingZoneButton(true);
			}
		}//GMRF
	} //Calcul pattern feature and display it

	/*******************************************************/
	/***Compare the research zone feature to the patern's***/
	/*******************************************************/
	else {
		//LBP METHOD
		if(toolsDock->getTextureMethod() == 0) {	
			m_openedFiles[m_workZoneTab->currentIndex()]->setClassifiedMat(ProcessingFunctions::LBPClassifier(this,m_openedFiles[m_workZoneTab->currentIndex()]->getModifiedMat(),init,end, m_lbpFeatures,m_lbpFeatures,toolsDock->getColorsChoice(),neighborhood,middle,m_testImageAction->isChecked()));
			toolsDock->desactivateSelectionZoneButton();		
		} //LBP
		
		//GMRF METHOD
		else if (toolsDock->getTextureMethod() == 1) {	
			m_openedFiles[m_workZoneTab->currentIndex()]->setClassifiedMat(ProcessingFunctions::GMRFClassifier(this,m_openedFiles[m_workZoneTab->currentIndex()]->getModifiedMat(),init,end, m_featuresMean,m_featuresMatrix,toolsDock->getColorsChoice(),neighborhood,middle,m_testImageAction->isChecked()));
			toolsDock->desactivateSelectionZoneButton();		
		} //MGRF

	} //Comparison
}
void MainWindow::selectionSearchingZone(bool checked){
	m_zoomInAction->setChecked(false);
	m_zoomOutAction->setChecked(false);
	m_zoomZoneAction->setChecked(false);
	if (m_openedFiles.size()>0){
		m_openedFiles[m_workZoneTab->currentIndex()]->getGraphicsView()->activeSearchingSelection(checked);
	}
}
void MainWindow::resetMat(){
	m_features.clear();
	m_featuresMatrix.clear();
	m_featuresMean.clear();
	m_lbpFeatures.clear();
	m_openedFiles[m_workZoneTab->currentIndex()]->resetMat();
	m_openedFiles[m_workZoneTab->currentIndex()]->getGraphicsView()->deleteRubbers();
	toolsDock->enableSearchingZoneButton(false);
	toolsDock->enableResetButtonMGRF(false);
}
int MainWindow::fileAlreadyOpened(QString type,int sd_index,QString fileName){
	bool isAlreadyOpened(false);
	int index(0);

	if(type=="SDS"){
		for (int i(0); i< m_openedFiles.size();i++){
			if(m_openedFiles[i]->getType() == "SDS"){
				if(sd_index==m_openedFiles[i]->getID() && fileName == m_openedFiles[i]->getFileName()){
						isAlreadyOpened = true;
						index = i; //The index of the tab where the sds is already diplayed
				}
			}
		}
	
		if (!isAlreadyOpened) index = -1; //If it's not already display we return -1
		else{
			//To have the right index of the m_workzonetab we add the number of vdatas before the sds wanted
			for (int i(0);i<=index;i++){
				if(i<m_openedFiles.size()){
					if(m_openedFiles[i]->getType()=="RGB"){
						index++;
					}
				}
			}
		}
	 
	}//SDS

	return index;
}
void MainWindow::allImageZone(){
	int row = m_openedFiles[m_workZoneTab->currentIndex()]->getMat().rows;
	int col = m_openedFiles[m_workZoneTab->currentIndex()]->getMat().cols;
	textureCalculSlot(QPointF(0,0),QPointF(row,col));
}
void MainWindow::openActionImageSlot(){
	QString fileName(QFileDialog::getOpenFileName(this,"Open a picture : ","C:/Users/Nico/Downloads/", "Images (*.png *.jpg *.jpeg *.bmp *.tif)"));//Open the Dialog window to open a file
	m_openedFiles.push_back(new OpenedFile(this,"Image",fileName));
	m_workZoneTab->addTab(m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(), fileName);
	m_workZoneTab->setCurrentIndex(m_openedFiles.size()-1);

	//Zoom Options
		QObject::connect(m_zoomInAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomIn(bool)));
		QObject::connect(m_zoomOutAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomOut(bool)));
		QObject::connect(m_zoomZoneAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomZone(bool)));
		QObject::connect(m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SIGNAL(paternCoordinates(QPointF,QPointF)),this,SLOT(textureCalculSlot(QPointF,QPointF)));


		if (m_zoomInAction->isChecked()){
			emit m_zoomInAction->toggled(true);
		}
		if (m_zoomOutAction->isChecked()){
			emit m_zoomOutAction->toggled(true);
		}
		if (m_zoomZoneAction->isChecked()){
			emit m_zoomZoneAction->toggled(true);
		}
		//Contrast and Luminosity
		toolsDock->enableSliders();
		
	

		m_workZoneTab->setCurrentIndex(m_openedFiles.size()-1);

}
void MainWindow::resetMatLBP(){
	if (toolsDock->getTextureMethod()==0){
		resetMat();
		toolsDock->resetTable();
	}
}
void MainWindow::testImageSlot(){
	//m_openedFiles.push_back(new OpenedFile(ProcessingFunctions::refLabel().clone()));
	m_openedFiles.push_back(new OpenedFile(this,"Image","C:/Users/Nico/Downloads/mstar/mstar/patch512_8b.tif"));
	m_workZoneTab->addTab(m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(), "Test");
	m_workZoneTab->setCurrentIndex(m_openedFiles.size()-1);

	//Zoom Options
		QObject::connect(m_zoomInAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomIn(bool)));
		QObject::connect(m_zoomOutAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomOut(bool)));
		QObject::connect(m_zoomZoneAction,SIGNAL(toggled(bool)),m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SLOT(activeZoomZone(bool)));
		QObject::connect(m_openedFiles[m_openedFiles.size()-1]->getGraphicsView(),SIGNAL(paternCoordinates(QPointF,QPointF)),this,SLOT(textureCalculSlot(QPointF,QPointF)));


		if (m_zoomInAction->isChecked()){
			emit m_zoomInAction->toggled(true);
		}
		if (m_zoomOutAction->isChecked()){
			emit m_zoomOutAction->toggled(true);
		}
		if (m_zoomZoneAction->isChecked()){
			emit m_zoomZoneAction->toggled(true);
		}
		//Contrast and Luminosity
		toolsDock->enableSliders();
		
	

		m_workZoneTab->setCurrentIndex(m_openedFiles.size()-1);
}

