#include "OpenedFile.h"

using namespace cv;
OpenedFile::OpenedFile(QMainWindow *context,QString type,QString fileName,int sd_id,uint16 **data,int32 *dimsize):m_type(type),m_fileName(fileName),m_sd_id(sd_id),m_context(context),m_contrast(0),m_luminosity(0),m_dimsize(dimsize){
	m_mat = uint16ToMat(data,dimsize,"GrayScale");	
	m_modifiedMat = m_mat.clone();
	m_classifiedMat = m_mat.clone();
	m_scene = createGraphicsScene(matToQImage(m_mat));
	m_view = createGraphicsView(m_scene,dimsize);
}
OpenedFile::OpenedFile(QMainWindow *context,QString type, uint16 **dataR,uint16 **dataG, uint16 **dataB, int32 *dimsize):m_type(type),m_context(context),m_contrast(0),m_luminosity(0),m_dimsize(dimsize){ //For RGB
	m_mat = toRGBMat(uint16ToMat(dataR,m_dimsize,"red"),uint16ToMat(dataG,m_dimsize,"green"),uint16ToMat(dataB,m_dimsize,"blue"),m_dimsize);
	m_modifiedMat = m_mat.clone();
	m_scene = createGraphicsScene(matToQImage(m_mat));
	m_view = createGraphicsView(m_scene,m_dimsize);
}
OpenedFile::OpenedFile(QMainWindow *context,QString type, QString fileName):m_context(context),m_type(type),m_fileName(fileName){
	m_mat = imread(m_fileName.toStdString());//save file into a Mat structure
	Mat matGrey = Mat(m_mat.rows,m_mat.cols, CV_8UC3);
	cv::cvtColor(m_mat, m_mat, CV_BGR2RGB); //conversion BGR to RGB
	for (int i(0);i<m_mat.rows;i++){
		for (int j(0);j<m_mat.cols;j++){
			matGrey.at<Vec3b>(i,j)[0] = m_mat.at<Vec3b>(i,j)[0];
			matGrey.at<Vec3b>(i,j)[1] = m_mat.at<Vec3b>(i,j)[0];
			matGrey.at<Vec3b>(i,j)[2] = m_mat.at<Vec3b>(i,j)[0];
		}

	}
	m_mat = matGrey.clone();
	m_modifiedMat = m_mat.clone();

	const uchar *qImageBuffer = (const uchar*)m_mat.data; //conversion from Mat to uchar*
	m_scene = createGraphicsScene(QImage(qImageBuffer, m_mat.cols, m_mat.rows, m_mat.step, QImage::Format_RGB888)); //conversion from uchar* to QImage
	int32 dimsize[2] = {m_mat.rows,m_mat.cols};
	m_view = createGraphicsView(m_scene,dimsize);
	
}
OpenedFile::OpenedFile(cv::Mat mat):m_mat(mat){
	int32 dim[2];
	dim[0] = 512;
	dim[1] = 512;
	m_modifiedMat = m_mat.clone();
	m_scene = createGraphicsScene(matToQImage(mat));
	m_view = createGraphicsView(m_scene,dim);
}
Mat OpenedFile::uint16ToMat(uint16 **data,int32 dimsize[2],QString band){
	QLabel *bandLabel;
	int numberOfSteps;
	if ( band == "GrayScale"){
	}
	else if (band == "red"){
		bandLabel = new QLabel("Red Band");
		m_context->statusBar()->addPermanentWidget(bandLabel);
	}
	else if (band == "green"){
		bandLabel = new QLabel("Green Band");
		m_context->statusBar()->addPermanentWidget(bandLabel);
	}
	else if (band == "blue"){
		bandLabel = new QLabel("Blue Band");
		m_context->statusBar()->addPermanentWidget(bandLabel);
	}
	m_taskLabel = new QLabel("1/4");
	m_progressBar = new QProgressBar();
	m_progressBar->setValue(0);
	m_progressBar->setMaximumHeight(15);
	m_progressBar->setMaximumWidth(150);
	m_progressBar->setAlignment(Qt::AlignRight);
	m_context->statusBar()->addPermanentWidget(m_taskLabel);
	m_context->statusBar()->addPermanentWidget(m_progressBar);
	Mat mat = Mat(dimsize[0],dimsize[1], CV_8UC3);

	//Calcul of the maximum
	uint16 max (0);
	for (int i(0);i<dimsize[0];i++){
		for (int j(0); j< dimsize[1];j++){
			if (data[i][j] > max){
				max = data[i][j];	
			}
		}		
		m_taskLabel->setText("1/4");
		m_progressBar->setValue(int(((float)i*100.)/(float)dimsize[0]));
	}

	//From 16 bits to 8 bits
	for (int i(0);i<dimsize[0];i++){
		for (int j(0); j< dimsize[1];j++){
			mat.at<Vec3b>(i,j)[0] = (uchar)((data[i][j])*255/max);
			mat.at<Vec3b>(i,j)[1] = (uchar)(data[i][j]*255/max);
			mat.at<Vec3b>(i,j)[2] = (uchar)(data[i][j]*255/max);
		}
		m_taskLabel->setText("2/4");		
		m_progressBar->setValue(int(((float)i*100.)/(float)dimsize[0]));
	}

	//Calcul of the minimum
	uchar min (255);
	for (int i(0);i<dimsize[0];i++){
		for (int j(0); j< dimsize[1];j++){
			if (mat.at<Vec3b>(i,j)[0] < min && mat.at<Vec3b>(i,j)[0] != 0){
				min = mat.at<Vec3b>(i,j)[0];
			}
		}
		m_taskLabel->setText("3/4");
		m_progressBar->setValue(int(((float)i*100.)/(float)dimsize[0]));

	}

	//And we put the mimimum at 0 to have a better contrast
	for (int i(0);i<dimsize[0];i++){
		for (int j(0); j< dimsize[1];j++){
			if (mat.at<Vec3b>(i,j)[0] !=0){
				mat.at<Vec3b>(i,j)[0] =(uchar)((float)(mat.at<Vec3b>(i,j)[0]-min)/(255-min)*255);
				mat.at<Vec3b>(i,j)[1] =(uchar)((float)(mat.at<Vec3b>(i,j)[1]-min)/(255-min)*255);
				mat.at<Vec3b>(i,j)[2] =(uchar)((float)(mat.at<Vec3b>(i,j)[2]-min)/(255-min)*255);
			}
		}
		m_taskLabel->setText("4/4");
		m_progressBar->setValue(int(((float)i*100.)/(float)dimsize[0]));


	}
	m_context->statusBar()->removeWidget(m_progressBar);
	m_context->statusBar()->removeWidget(bandLabel);
	m_context->statusBar()->removeWidget(m_taskLabel);

	return mat;


}
QImage OpenedFile::matToQImage(cv::Mat mat){
	const uchar *qImageBuffer = (const uchar*)mat.data; //conversion from Mat to uchar*
	QImage image = QImage(qImageBuffer,mat.cols, mat.rows,mat.step, QImage::Format_RGB888); //conversion from uchar* to QImage
	return image;
}
QGraphicsScene* OpenedFile::createGraphicsScene(QImage image){
	QGraphicsScene *scene = new QGraphicsScene();
	scene->addPixmap(QPixmap::fromImage(image));
	return scene;
}
MyGraphicsView* OpenedFile::createGraphicsView(QGraphicsScene* scene,int32 *dimsize){
	MyGraphicsView *view = new MyGraphicsView(dimsize[0],dimsize[1]);
	view->setScene(scene);
	return view;
}
MyGraphicsView* OpenedFile::getGraphicsView(){
	return m_view;
}
int OpenedFile::getID(){
	return m_sd_id;
}
QString OpenedFile::getFileName(){
	return m_fileName;
}
QString OpenedFile::getType(){
	return m_type;
}
void OpenedFile::modifyMat(int contrast, int luminosity,cv::Mat mat){
	m_contrast = contrast;
	m_luminosity = luminosity;
	m_modifiedMat = mat.clone();
	m_scene = createGraphicsScene(matToQImage(mat));
	m_view->setScene(m_scene);
}
cv::Mat OpenedFile::getMat(){
	return m_mat;
}
int OpenedFile::getContrast(){
	return m_contrast;
}
int OpenedFile::getLuminosity(){
	return m_luminosity;

}
Mat OpenedFile::toRGBMat(Mat matR,Mat matG,Mat matB,int32 *dimsize){
	Mat matRGB = Mat(dimsize[0],dimsize[1], CV_8UC3);
	for (int i(0);i<dimsize[0];i++){
		for (int j(0);j<dimsize[1];j++){
			matRGB.at<Vec3b>(i,j)[0] = matR.at<Vec3b>(i,j)[0];
			matRGB.at<Vec3b>(i,j)[1] = matG.at<Vec3b>(i,j)[0];
			matRGB.at<Vec3b>(i,j)[2] = matB.at<Vec3b>(i,j)[0];
		}
	}

	return matRGB;
}
cv::Mat OpenedFile::getModifiedMat(){
	return m_modifiedMat;
}
void OpenedFile::setClassifiedMat(cv::Mat mat){
	m_classifiedMat = mat.clone();
	m_scene = createGraphicsScene(matToQImage(mat));
	m_view->setScene(m_scene);
}
void OpenedFile::resetMat(){
	m_scene = createGraphicsScene(matToQImage(m_modifiedMat));
	m_view->setScene(m_scene);
}