#include "ProcessingFunctions.h"


using namespace cv;
cv::Mat ProcessingFunctions::contrast(QMainWindow *context,cv::Mat matImage, double alpha,double beta){
	
	QProgressBar *progressBar = new QProgressBar();
	progressBar->setValue(0);
	progressBar->setMaximumHeight(15);
	progressBar->setMaximumWidth(150);
	progressBar->setAlignment(Qt::AlignRight);
	context->statusBar()->addPermanentWidget(progressBar);
	
	Mat modifiedImage = Mat::zeros( matImage.size(), matImage.type() );

	for( int y = 0; y < matImage.rows; y++ ){ 
		for( int x = 0; x < matImage.cols; x++ ){
			for( int c = 0; c < 3; c++ )
             {
				modifiedImage.at<Vec3b>(y,x)[c] =
				saturate_cast<uchar>( alpha*( matImage.at<Vec3b>(y,x)[c]+beta));
             }
			progressBar->setValue(int(((float)y*100.)/(float)matImage.rows));
		}
	}
	context->statusBar()->removeWidget(progressBar);
	return modifiedImage;
}
QVector<double> ProcessingFunctions::LBPCalculRef(Mat modifiedMat,QPointF init,QPointF end, int neighborhood,int middle){
	QVector<int> vectorBuffer;
	QVector<int> vectorBufferRotate;
	QVector<double> histogram;
	int increment(0);
	long long indexBuffer(0);
	int dim1 = (int)end.x()-(int)init.x();
	int dim2 = (int)end.y()-(int)init.y();

	histogram.resize(pow(2,16)+1);
	for (int i((int)init.x()+3);i<(int)init.x()+dim1-5;i++){ //cols
		for (int j((int)init.y()+3); j<(int)init.y()+dim2-5;j++){	//raws		
			histogram[lbpIndexCalcul(j,i,neighborhood,middle, modifiedMat)]++;
		}
	}
	double sum(0.);
	//Normalisation of the histogram (the integral = 1)
	for (int i(0);i<(int)pow(2,16)+1;i++){
		histogram[i] = histogram[i]/(double)((dim1-8)*(dim2-8));
		sum +=histogram[i];
	}
int b = histogram[pow(2,16)];
	return histogram;


}
void ProcessingFunctions::GMRFCalculRefMean(cv::Mat mat,int neighborhood,double *mean){
	int dim0(mat.rows),dim1(mat.cols),sum(0);
	double variance(0),squareSum(0);

	for (int i(0);i<(int)pow(neighborhood,2);i++){
		mean[i] = 0.;
	}
	int increment(0);
	//Mean
	for (int i(1);i<dim0-(neighborhood-1);i = i+neighborhood){
		for (int j(1);j<dim1-(neighborhood-1);j = j+neighborhood){
			increment = 0;
			for (int k(i-1);k<i-1+neighborhood;k++){
				for (int l(j-1);l<j-1+neighborhood;l++){
					mean[increment] += mat.at<Vec3b>(k,l)[0];
					increment++;
				}
			}
		}
	}
	double test;
	for (int i(0) ; i<(int)pow(neighborhood,2);i++){
		mean[i] /= ((((double)dim1-2.))*((double)dim0-2.))/pow(neighborhood,2);
		test = mean[i];
	}
	
}
void ProcessingFunctions::GMRFCalculRefMatrix(cv::Mat mat,int neighborhood,double* mean,double **inverseMatrix){
	int dim0(mat.rows),dim1(mat.cols);
	double test;

	double *vectorBuffer = new double[(int)pow(neighborhood,2)];
	for (int i(0);i<(int)pow(neighborhood,2);i++){
		vectorBuffer[i] = 0.;
	}
	double** matrixBuffer = new double*[(int)pow(neighborhood,2)];
	for (int i(0);i<(int)pow(neighborhood,2);i++){
		matrixBuffer[i] = new double[(int)pow(neighborhood,2)];
		for (int j(0);j<pow(neighborhood,2);j++){
			matrixBuffer[i][j] = 0;
		}
	}
	
	double** precisionMatrix = new double*[(int)pow(neighborhood,2)];
	for (int i(0);i<(int)pow(neighborhood,2);i++){
		precisionMatrix[i] = new double[(int)pow(neighborhood,2)];
		for (int j(0);j<pow(neighborhood,2);j++){
			precisionMatrix[i][j] = 0;
		}
	}

	int increment(0);
	for (int i(1);i<dim0-1;i=i+neighborhood){
		for (int j(1);j<dim1-1;j=j+neighborhood){
			increment = 0;
			for (int k(i-1);k<i-1+neighborhood;k++){
				for (int l(j-1);l<j-1+neighborhood;l++){	
					vectorBuffer[increment] = (double)mat.at<Vec3b>(k,l)[0]-mean[increment];
					increment++;
				}
			}
			for (int i(0);i<(int)pow(neighborhood,2);i++){
				for (int j(0);j<(int)pow(neighborhood,2);j++){
					matrixBuffer[i][j] = vectorBuffer[i]*vectorBuffer[j];
				}
			}
			
			for (int k(0);k<(int)pow(neighborhood,2);k++){
				for (int l(0);l<(int)pow(neighborhood,2);l++){
					precisionMatrix[k][l] += matrixBuffer[k][l];
					
				}
			}
		}
	}



	for (int i(0);i<(int)pow(neighborhood,2);i++){
		for (int j(0);j<(int)pow(neighborhood,2);j++){
			if (precisionMatrix[i][j]<0) precisionMatrix[i][j] = -precisionMatrix[i][j];
			precisionMatrix[i][j]/=(((double)dim0-2)*((double)dim1-2))/pow(neighborhood,2);
		}
	}

	MatrixFunctions::inverseMatrix(precisionMatrix,pow(neighborhood,2),inverseMatrix);



	delete [] vectorBuffer;
	delete [] matrixBuffer;
	delete [] precisionMatrix;
}
cv::Mat ProcessingFunctions::LBPClassifier(QMainWindow *context,Mat modifiedMat,QPointF init,QPointF end,QVector<QVector<double>> features,QVector<QVector<double>> featuresNormal, bool colorsType,int neighborhood,int middle,bool test){
	//Variable declarations
	int dim1(0),dim2(0),label(0),d(0),sumRightLabel(0);
	int *vector,*vectorColors,*neighborhoodAndMiddle;
	double diff(0.), min(0.),performance(0.);
	Mat matZone, matBuffer;
	int** matrixLabel = ProcessingFunctions::createMatrixLabel();
	QVector<QVector<int>> index;
	QVector<double> histogramBuffer(pow(2,16)+1);

	index.resize(modifiedMat.cols);
	for (int i (0);i<modifiedMat.cols;i++){
		index[i].resize(modifiedMat.rows);
	}

	dim1 = (int)end.x()-(int)init.x();
	dim2 = (int)end.y()-(int)init.y();

	vector = new int[16];
	
	matBuffer = modifiedMat.clone();

	//Progress Bar in the Main Window
	QProgressBar *progressBar = new QProgressBar();
	progressBar->setValue(0);
	progressBar->setMaximumHeight(15);
	progressBar->setMaximumWidth(150);
	progressBar->setAlignment(Qt::AlignRight);
	context->statusBar()->addPermanentWidget(progressBar);


	//Begin of the loop among each pixel of the research zone
	for (int i((int)init.x());i<(int)init.x()+dim1-1;i++){ //cols
		for (int j((int)init.y()); j<(int)init.y()+dim2-1;j++){	//rows		
			index[j][i] = lbpIndexCalcul(j,i,neighborhood,middle, modifiedMat); //j = rows; i = cols
			progressBar->setValue(int(((float)(i-init.x())*100.)/(float)dim1));
		}
	} //End of the loop

	for (int i((int)init.x()+3);i<(int)init.x()+dim1-5;i++){ //cols
		for (int j((int)init.y()+3); j<(int)init.y()+dim2-5;j++){	//raws		
				
			for (int l(0);l<pow(2,15)+1;l++){
					histogramBuffer[l] = 0;
				}
			histogramBuffer[pow(2,16)] = 0;
					
			for (int k(j-3);k<j+4;k++){
				for (int l(i-3);l<i+4;l++){
					histogramBuffer[index[k][l]]++;			
				}
			}

			min = 10000000;
			int sum(0);
			//We calcul the "difference" between the histogramBuffer and the histogram of each label
			for (int k(0);k<features.size();k++){
				diff = 0.;
				sum = 0;
				for (int l(0);l<pow(2,15)+1;l++){
					if (features[k][l]!=0.){
						diff +=	abs((double)histogramBuffer[l]-(double)features[k][l])/features[k][l];
						sum++;
					}
					else {
						if (histogramBuffer[l] !=0. ) {
							diff += abs((double)histogramBuffer[l]-(double)features[k][l]);
							sum++;
						}
					}
				}
					if(features[k][pow(2,16)]!=0.){
						diff +=	abs((double)histogramBuffer[pow(2,16)]-(double)features[k][pow(2,16)])/features[k][pow(2,16)];
						sum++;
					}
					else {
						if (histogramBuffer[pow(2,16)] !=0.){
								diff += abs((double)histogramBuffer[pow(2,16)]-(double)features[k][pow(2,16)]);
								sum++;
						}
					}
					diff/=sum;
				if (diff<min){
					min = diff;
					label = k;
				}
			}
			
			//Measure Performance
			//if(test) {
			//	if (label == matrixLabel[j+middle][i+middle]){
			//		sumRightLabel++;
			//	}

			//}

			//We "colorize" the pixel depending on its label
			vectorColors = switchColorize(label,colorsType);
			if (colorsType){
				matBuffer.at<Vec3b>(j+middle,i+middle)[0]=saturate_cast<uchar>(matBuffer.at<Vec3b>(j+middle,i+middle)[0]+vectorColors[0]);
				matBuffer.at<Vec3b>(j+middle,i+middle)[1]=saturate_cast<uchar>(matBuffer.at<Vec3b>(j+middle,i+middle)[1]+vectorColors[1]);
				matBuffer.at<Vec3b>(j+middle,i+middle)[2]=saturate_cast<uchar>(matBuffer.at<Vec3b>(j+middle,i+middle)[2]+vectorColors[2]);

			}
			else{
				matBuffer.at<Vec3b>(j+middle,i+middle)[0]=saturate_cast<uchar>(vectorColors[0]);
				matBuffer.at<Vec3b>(j+middle,i+middle)[1]=saturate_cast<uchar>(vectorColors[1]);
				matBuffer.at<Vec3b>(j+middle,i+middle)[2]=saturate_cast<uchar>(vectorColors[2]);
			}
			progressBar->setValue(int(((float)(i-init.x())*100.)/(float)dim1));
		}
	}
	//	//Begin of the loop among each pixel of the research zone
	//for (int i((int)init.x()+3);i<(int)init.x()+dim1-5;i++){ //cols
	//	for (int j((int)init.y()+3); j<(int)init.y()+dim2-5;j++){	//raws		
	//		for (int k(j-1);k<j+2;k++){
	//			for (int l(i-1);l<i+2;l++){
	//				histogramBuffer[lbpIndexCalcul(k,l,neighborhood,middle, modifiedMat)]++;
	//			}
	//		}

	//		for (int m(0);m<pow(2,16);m++){
	//			histogramBuffer[m] = histogramBuffer[m]/(9);
	//		}

	//		min = 10000;
	//		//We calcul the "difference" between the histogramBuffer and the histogram of each label
	//		for (int k(0);k<features.size();k++){
	//			diff = 0;
	//			for (int l(0);l<pow(2,16);l++){
	//					diff +=	abs((double)histogramBuffer[l]-(double)features[k][l]);
	//			}
	//			if (diff<=min){
	//				min = diff;
	//				label = k;
	//			}
	//		}
	//		
	//		//Measure Performance
	//		if(test) {
	//			if (label == matrixLabel[j+middle][i+middle]){
	//				sumRightLabel++;
	//			}

	//		}

	//		//We "colorize" the pixel depending on its label
	//		vectorColors = switchColorize(label,colorsType);
	//		if (colorsType){
	//			matBuffer.at<Vec3b>(j+middle,i+middle)[0]=saturate_cast<uchar>(matBuffer.at<Vec3b>(j+middle,i+middle)[0]+vectorColors[0]);
	//			matBuffer.at<Vec3b>(j+middle,i+middle)[1]=saturate_cast<uchar>(matBuffer.at<Vec3b>(j+middle,i+middle)[1]+vectorColors[1]);
	//			matBuffer.at<Vec3b>(j+middle,i+middle)[2]=saturate_cast<uchar>(matBuffer.at<Vec3b>(j+middle,i+middle)[2]+vectorColors[2]);

	//		}
	//		else{
	//			matBuffer.at<Vec3b>(j+middle,i+middle)[0]=saturate_cast<uchar>(vectorColors[0]);
	//			matBuffer.at<Vec3b>(j+middle,i+middle)[1]=saturate_cast<uchar>(vectorColors[1]);
	//			matBuffer.at<Vec3b>(j+middle,i+middle)[2]=saturate_cast<uchar>(vectorColors[2]);
	//		}
	//		progressBar->setValue(int(((float)(i-init.x())*100.)/(float)dim1));
	//	}
	//} //End of the loop
	//performance = (double)sumRightLabel/(double)((dim1-3)*(dim2-3));
	//QMessageBox::information(context, "Performance", "Performance = "+QString::number(performance*100,'g',4)+"%");	
	context->statusBar()->removeWidget(progressBar);
	return matBuffer;
}
Mat ProcessingFunctions::GMRFClassifier(QMainWindow *context,cv::Mat modifiedMat,QPointF init,QPointF end, QVector<double*> featuresMean, QVector<double**> featuresMatrix, bool colorsType,int neighborhood,int middle,bool test){
	double beta (0.1),energyBuffer(0),energyMin(10000),performance(0);
	int valueBuffer(0),sumRightLabel(0),dim1((int)end.x()-(int)init.x()),dim2((int)end.y()-(int)init.y());
	int *vectorColors, *neighborhoodAndMiddle;
	int** matrixLabel = ProcessingFunctions::createMatrixLabel();
	Mat matNeighBors(3,3,CV_8UC3),matBuffer(modifiedMat.clone()),matZone(Mat(dim2,dim1,CV_8UC3));
	double *vectorValueBuffer;
	double testm;

	vectorValueBuffer = new double[(int)pow(neighborhood,2)];
	//Mat structure of the research zone
	for (int i(0);i<dim2;i++){
			for (int j(0); j<dim1;j++){
					matZone.at<Vec3b>(i,j)[0] =modifiedMat.at<Vec3b>(i+(int)init.y(),j+(int)init.x())[0];
			}
	}

	//Containing labels of each pixel
	int **labelNumber;

	//Initialize dynamical tables
	labelNumber = new int*[modifiedMat.cols];
	for (int i(0);i<modifiedMat.cols;i++){
		labelNumber[i] = new int[modifiedMat.rows];
		for (int j(0); j<modifiedMat.rows;j++){
			labelNumber[i][j] = 0;
		}
	}
	int** labelNumber2 = new int*[modifiedMat.cols];
	for (int i(0);i<modifiedMat.cols;i++){
		labelNumber2[i] = new int[modifiedMat.rows];
		for (int j(0); j<modifiedMat.rows;j++){
			labelNumber2[i][j] = 0;
		}
	}



	//Progress Bar in the Main Window
	QProgressBar *progressBar = new QProgressBar();
	progressBar->setValue(0);
	progressBar->setMaximumHeight(15);
	progressBar->setMaximumWidth(150);
	progressBar->setAlignment(Qt::AlignRight);
	context->statusBar()->addPermanentWidget(progressBar);
		double test(0);
		int increment(0);

		/****************************************************************/
		/********Mi intento para hacer el clasificador bayesiano*********/
		/****************************************************************/
		
		//double det[2]; 
		//for (int m(0);m<2;m++){
		//	det[m] = (abs(MatrixFunctions::CalcDeterminant(featuresMatrix[m],pow(neighborhood,2))));
		//}
		
		
	//Begining of the loop along each pixel of the image
	//for (int i((int)init.x());i<(int)init.x()+dim1-3;i++){
	//	for (int j((int)init.y()); j<(int)init.y()+dim2-3;j++){
	//		energyMin = 100000;
	//		//We calculate the energy for each reference feature
	//		for (int m(0);m<featuresMean.size();m++){ 
	//			energyBuffer = 0;
	//			increment = 0;
	//			for (int k(j);k<j+neighborhood;k++){
	//				for (int l(i);l<i+neighborhood;l++){
	//					vectorValueBuffer[increment] = (double)modifiedMat.at<Vec3b>(k,l)[0]-featuresMean[m][increment];
	//					test = featuresMean[m][increment];
	//					test = vectorValueBuffer[increment];
	//					increment++;
	//				}
	//			}
	//			energyBuffer =log(det[m])+MatrixFunctions::multiple(vectorValueBuffer,featuresMatrix[m],(int)pow(neighborhood,2));			
	//			//test = log(abs(MatrixFunctions::CalcDeterminant(featuresMatrix[m],pow(neighborhood,2))));
	//			//And we find the feature which minimize the energy
	//			if(energyBuffer<energyMin){
	//				energyMin= energyBuffer;
	//				labelNumber[j+middle][i+middle] =  m;
	//			}
	//		} //Each feature
	//	}
	//}
	//double d(0.);
	//for (int i((int)init.x());i<(int)init.x()+dim1-3;i++){
	//	for (int j((int)init.y()); j<(int)init.y()+dim2-3;j++){
	//		energyMin = 100000;
	//		for (int m(0);m<featuresMean.size();m++){ 
	//			energyBuffer = 0;
	//			increment = 0;
	//			for (int k(j);k<j+neighborhood;k++){
	//				for (int l(i);l<i+neighborhood;l++){
	//					if (m==labelNumber[k][l]) d = -beta;
	//					else d = beta;
	//					valueBuffer = modifiedMat.at<Vec3b>(k,l)[0];
	//					vectorValueBuffer[increment] = (double)modifiedMat.at<Vec3b>(k,l)[0]-featuresMean[m][increment];
	//					increment++;
	//				}
	//			}
	//			energyBuffer = log(det[m])+MatrixFunctions::multiple(vectorValueBuffer,featuresMatrix[m],(int)pow(neighborhood,2)) + d ;				

	//			//And we find the feature which minimize the energy
	//			if(energyBuffer<energyMin){
	//				energyMin= energyBuffer;
	//				labelNumber2[j+middle][i+middle] =  m;
	//			}
	//		} //Each feature
			double diff(0.);
			double minDiff(100000.);
			for (int i((int)init.x());i<(int)init.x()+dim1-3;i++){
				for (int j((int)init.y()); j<(int)init.y()+dim2-3;j++){
					minDiff = 100000.;
					for (int m(0);m<featuresMean.size();m++){ 
						diff = 0.;
						increment = 0;
						for (int k(j);k<j+neighborhood;k++){
							for (int l(i);l<i+neighborhood;l++){
								diff += featuresMean[m][increment] - modifiedMat.at<Vec3b>(k,l)[0];
								increment++;
							}
						}
						if (abs(diff)<minDiff){
							minDiff = abs(diff);
							labelNumber2[j+middle][i+middle] = m;
						}
					}
			
			
			//Measure Performance
			if(test) {
				if (labelNumber2[j+middle][i+middle] == matrixLabel[j+middle][i+middle]){
					sumRightLabel++;
				}

			}

			//And we "colorize" the pixel depending on the label
			vectorColors = switchColorize(labelNumber2[j+middle][i+middle],colorsType);
			if (colorsType){
				matBuffer.at<Vec3b>(j+middle,i+middle)[0]=saturate_cast<uchar>(matBuffer.at<Vec3b>(j+middle,i+middle)[0]+vectorColors[0]);
				matBuffer.at<Vec3b>(j+middle,i+middle)[1]=saturate_cast<uchar>(matBuffer.at<Vec3b>(j+middle,i+middle)[1]+vectorColors[1]);
				matBuffer.at<Vec3b>(j+middle,i+middle)[2]=saturate_cast<uchar>(matBuffer.at<Vec3b>(j+middle,i+middle)[2]+vectorColors[2]);

			}
			else{
				matBuffer.at<Vec3b>(j+middle,i+middle)[0]=saturate_cast<uchar>(vectorColors[0]);
				matBuffer.at<Vec3b>(j+middle,i+middle)[1]=saturate_cast<uchar>(vectorColors[1]);
				matBuffer.at<Vec3b>(j+middle,i+middle)[2]=saturate_cast<uchar>(vectorColors[2]);
			}

			progressBar->setValue(int(((float)(i-init.x())*100.)/(float)dim1));	
		}	
	}//End of the loop

	performance = (double)sumRightLabel/(double)((dim1-3)*(dim2-3));
	QMessageBox::information(context, "Performance", "Performance = "+QString::number(performance*100,'g',4)+"%");	
	context->statusBar()->removeWidget(progressBar);
	return matBuffer;

	delete [] matrixLabel;
	delete [] labelNumber;
	delete [] labelNumber2;
}
Mat ProcessingFunctions::refLabel(){
	Mat mat = Mat(512,512, CV_8UC3);
	int r(129);
	for (int i(0);i<127;i++){
		for (int j(0); j<255;j++){
			mat.at<Vec3b>(i,j)[0] = 255;
			mat.at<Vec3b>(i,j)[1] = 0;
			mat.at<Vec3b>(i,j)[2] = 0;
		}
	}
	for (int i(127);i<255;i++){
		for (int j(0);j<256-(int)sqrt(pow(r,2)-pow(256-i,2));j++){
			mat.at<Vec3b>(i,j)[0] = 255;
			mat.at<Vec3b>(i,j)[1] = 0;
			mat.at<Vec3b>(i,j)[2] = 0;
		}
	}


	for (int i(254);i<382;i++){
		for (int j(0); j<256-(int)sqrt(pow(r,2)-pow(i-254,2));j++){
			mat.at<Vec3b>(i,j)[0] = 0;
			mat.at<Vec3b>(i,j)[1] = 255;
			mat.at<Vec3b>(i,j)[2] = 0;
		}
	}
	for (int i(382);i<511;i++){
		for (int j(0);j<256;j++){
			mat.at<Vec3b>(i,j)[0] = 0;
			mat.at<Vec3b>(i,j)[1] = 255;
			mat.at<Vec3b>(i,j)[2] = 0;
		}
	}

	for (int i(382);i<511;i++){
		for (int j(255); j<511;j++){
			mat.at<Vec3b>(i,j)[0] = 0;
			mat.at<Vec3b>(i,j)[1] = 0;
			mat.at<Vec3b>(i,j)[2] = 255;
		}
	}
	for (int i(255);i<382;i++){
		for (int j(256+(int)sqrt(pow(r,2)-pow(i-254,2)));j<511;j++){
			mat.at<Vec3b>(i,j)[0] = 0;
			mat.at<Vec3b>(i,j)[1] = 0;
			mat.at<Vec3b>(i,j)[2] = 255;
		}
	}
	for (int i(127);i<255;i++){
		for (int j(256+(int)sqrt(pow(r,2)-pow(256-i,2))); j<511;j++){
			mat.at<Vec3b>(i,j)[0] = 0;
			mat.at<Vec3b>(i,j)[1] = 255;
			mat.at<Vec3b>(i,j)[2] = 255;
		}
	}
	for (int i(0);i<127;i++){
		for (int j(255);j<511;j++){
			mat.at<Vec3b>(i,j)[0] = 0;
			mat.at<Vec3b>(i,j)[1] = 255;
			mat.at<Vec3b>(i,j)[2] = 255;
		}
	}
	return mat;
}
int** ProcessingFunctions::createMatrixLabel(){
	static int** matrixLabelRef;
	matrixLabelRef = new int*[512];
	for (int i(0) ; i<512 ; i++){
		matrixLabelRef[i] = new int[512];
	}

	int r(129);

	for (int i(0);i<512;i++){
		for (int j(0);j<512;j++){
			matrixLabelRef[i][j] = 4;
		}
	}
	for (int i(0);i<127;i++){
		for (int j(0); j<255;j++){
			matrixLabelRef[i][j] = 0;
		}
	}
	for (int i(127);i<255;i++){
		for (int j(0);j<256-(int)sqrt(pow(r,2)-pow(256-i,2));j++){
			matrixLabelRef[i][j] = 0;
		}
	}


	for (int i(254);i<382;i++){
		for (int j(0); j<256-(int)sqrt(pow(r,2)-pow(i-254,2));j++){
			matrixLabelRef[i][j] = 1;
		}
	}
	for (int i(382);i<512;i++){
		for (int j(0);j<256;j++){
			matrixLabelRef[i][j] = 1;
		}
	}

	for (int i(382);i<512;i++){
		for (int j(255); j<511;j++){
			matrixLabelRef[i][j] = 2;
		}
	}
	for (int i(255);i<382;i++){
		for (int j(256+(int)sqrt(pow(r,2)-pow(i-254,2)));j<511;j++){
			matrixLabelRef[i][j] = 2;
		}
	}
	for (int i(127);i<255;i++){
		for (int j(256+(int)sqrt(pow(r,2)-pow(256-i,2))); j<511;j++){
			matrixLabelRef[i][j] = 3;
		}
	}
	for (int i(0);i<127;i++){
		for (int j(255);j<511;j++){
			matrixLabelRef[i][j] = 3;
		}
	}

	return matrixLabelRef;


}
int* ProcessingFunctions::switchColorize(int label, bool colorsType){
	static int colors[3];
	for (int i(0);i<3;i++){
		colors[i] = 0;
	}

	if (colorsType){
		switch(label){
		case 0: //red
			colors[0] = 50;
			break;
		case 1://green
			colors[1] = 50;	
			break;
		case 2: //blue
			colors[2] = 50;
			break;
		case 3: //magenta
			colors[0] = 50;
			colors[2] = 50;
			break;
		case 4: //yellow
			colors[0] = 50;
			colors[1] = 50;
			break;
		case 5: //cyan
			colors[1] = 50;
			colors[2] = 50;
			break;
				
		case 6: //dark red
			colors[0] = 20;
			break;
				
		case 7: //dark green
			colors[1] = 20;
			break;
				
		case 8: //dark blue
			colors[2] = 20;
			break;
		}
	}

	else{
		switch(label){
		case 0: //red
			colors[0] = 255;
			break;
		case 1://green
			colors[1] = 255;
			break;
		case 2: //blue
			colors[2] = 255;
			break;
		case 3: //magenta
			colors[0] = 255;
			colors[2] = 255;
			break;
		case 4: //yellow
			colors[0] = 255;
			colors[1] = 255; 
			break;
		case 5: //cyan
			colors[1] = 255;
			colors[2] = 255;
			break;
			
		case 6: //dark red
			colors[0] = 128;
			break;
			
		case 7: //dark green
			colors[1] = 128;

			break;
			
		case 8: //dark blue
			colors[2] = 128;
			break;
		} //Switch
	}

	return colors;
}
int* ProcessingFunctions::neignborhoodAndMiddle(int neighborsChoice){
	static int neighborsAndMidle[2];
	switch(neighborsChoice){
	case 0:
		neighborsAndMidle[0] = 3;
		neighborsAndMidle[1] = 1;
		break;
	case 1:
		neighborsAndMidle[0] = 5;
		neighborsAndMidle[1] = 2;
		break;
	case 2:
		neighborsAndMidle[0] = 7;
		neighborsAndMidle[1] = 3;
		break;
	case 3:
		neighborsAndMidle[0] = 9;
		neighborsAndMidle[1] = 4;
		break;
	}
	return neighborsAndMidle;
}
double ProcessingFunctions::indexMeanCalcul(QVector<double> index,int neighborhood){
	double indexMean(0.);
	for(int i(0);i<pow(2,pow(neighborhood,2)-1);i++){
		indexMean+=i*index[i];
	}
	return indexMean;
}
int ProcessingFunctions::lbpIndexCalcul(int i, int j, int neighborhood,int middle, cv::Mat mat){ //i = rows ; j = cols
	QVector<int> vectorBuffer,vectorBufferRotate;
	int increment = 0;

    double  grayValues[16];
	for (int k(0);k<16;k++){
		grayValues[k] = 0;
	}
	QVector<QPointF> coorPoints;
	coorPoints.push_back(QPointF(i+2,j-1));
	coorPoints.push_back(QPointF(i+2,j));
	coorPoints.push_back(QPointF(i+3,j));
	coorPoints.push_back(QPointF(i+3,j+2));
	coorPoints.push_back(QPointF(i+2,j+2));
	coorPoints.push_back(QPointF(i+2,j+3));
	coorPoints.push_back(QPointF(i,j+3));
	coorPoints.push_back(QPointF(i,j+2));
	coorPoints.push_back(QPointF(i-1,j+2));
	coorPoints.push_back(QPointF(i-1,j));
	coorPoints.push_back(QPointF(i,j));
	coorPoints.push_back(QPointF(i,j-1));

	double theta(22.5);
	double distance(0.);
	double sumDistance(0.);
	int jump(0);
	for (int k(0);k<12;k++){
		sumDistance = 0.;
		if(theta == 90 || theta == 180 || theta == 270){
			theta +=22.5;
			jump++;
		}
		for(int m(0);m<3;m++){
			for (int n(0);n<3;n++){
				double a = 2*sin(22.5*2*M_PI/360);
				int b = coorPoints[k].y()+(m-1)-(j+1);
				distance = sqrt(pow(2*sin(theta*2*M_PI/360)-(coorPoints[k].x()+(n-1)-(i+1)),2)+ pow(-2*cos(theta*2*M_PI/360)-(coorPoints[k].y()+(m-1)-(j+1)),2));
				grayValues[k+jump] += distance*mat.at<Vec3b>(coorPoints[k].x()+m-1,coorPoints[k].y()+n-1)[0];//faux
				sumDistance += distance;
			}
		}
		grayValues[k+jump]/=sumDistance;
		theta+=22.5;
	}





	
	grayValues[0] = mat.at<Vec3b>(i+1,j-2)[0];
	grayValues[4] = mat.at<Vec3b>(i+3,j+1)[0];
	grayValues[8] = mat.at<Vec3b>(i+1,j+3)[0];
	grayValues[12] = mat.at<Vec3b>(i-1,j+1)[0];

	for (int k(0);k<16;k++){
		int a = mat.at<Vec3b>(i+middle,j+middle)[0];
		if (grayValues[k]-mat.at<Vec3b>(i+middle,j+middle)[0]>=0){
			vectorBuffer.push_back(1);
		}
		else{
			vectorBuffer.push_back(0);
		}
	}

	//for (int k(i);k<i+neighborhood;k++){
	//	for (int l(j);l<j+neighborhood;l++){
	//		if(increment!=(pow(neighborhood,2)-1)/2){//Avoid the center 
	//			if (mat.at<Vec3b>(i+middle,j+middle)[0]<=mat.at<Vec3b>(k,l)[0]){
	//				vectorBuffer.push_back(0);
	//			}
	//			else{
	//				vectorBuffer.push_back(1);
	//			}
	//		}
	//		increment++;
	//	}
	//}
	//Reduction of the index for rotation invariance


	int m(0);
	unsigned long long index(0), minIndex = pow(2.,16);
	for (int k(0);k<16;k++){
		vectorBufferRotate = vectorBuffer;

		//Rotation
		for (int l(0);l<16;l++){
			if((l-k)>=0){
				m = l-k;
			}
			else {
				m=15+(l-k); 
			}
			vectorBufferRotate[l] = vectorBuffer[m];
		}

		//Index calcul
		index = 0;
		for(int n(0);n<16;n++){
				index += vectorBufferRotate[n]*(int)pow(2,n);
		}
		if(index<minIndex){
			minIndex = index;
		}
	}
	return minIndex;


}
double* ProcessingFunctions::histogramToGaussian(double *histogram){
	static double normal[2];//{Mean,Variance};
	normal[0] = 0.;
	normal[1] = 0.;
	int numberOfPixels(0);
	//Mean
	for (int i(1);i<128;i++){
		normal[0] += i*(double)histogram[i];
	}

	//Variance
	for (int i(1);i<128;i++){
		normal[1]+=pow(((i*(double)histogram[i])-normal[0]),2);
	}
	normal[1] = sqrt(normal[1]);
	return normal;
}