#include "ToolsDock.h"

ToolsDock::ToolsDock(QString title):QDockWidget(title),m_automaticChange(false){

	setMinimumWidth(200);
	tabWidget = new QTabWidget();
	setWidget(tabWidget);
	setColorsSet();

	/////////ADJUSTMENTS////////
	QWidget *centralWidget = new QWidget();
	QVBoxLayout *layoutPrincipal = new QVBoxLayout();
	centralWidget->setLayout(layoutPrincipal);

	//Contrast
	QVBoxLayout *layoutContrast = new QVBoxLayout();
	QGroupBox *contrastBox = new QGroupBox("Contrast");
	contrastValue = new QLabel("0",contrastBox);
	contrastBox->setLayout(layoutContrast);
	contrastSlider = new MySlider(Qt::Horizontal);
	contrastSlider->setEnabled(false);
	QObject::connect(contrastSlider,SIGNAL(valueChanged(int)),this,SLOT(contrastValueChangedLabelSlot(int)));
	QObject::connect(contrastSlider,SIGNAL(valueModified(int)),this,SLOT(contrastValueChangedSlot(int)));
	layoutContrast->addWidget(contrastSlider);
	layoutContrast->addWidget(contrastValue);
	layoutContrast->setAlignment(contrastValue,Qt::AlignHCenter);
	contrastSlider->setFixedHeight(20);
	contrastBox->setFixedHeight(70);
	contrastSlider->setMaximum(100);
	contrastSlider->setMinimum(-100);
	

	//Luminosity
	QVBoxLayout *layoutLuminosity = new QVBoxLayout();
	QGroupBox *luminosityBox = new QGroupBox("Luminosity");
	luminosityValue = new QLabel("0",luminosityBox);
	luminosityBox->setLayout(layoutLuminosity);
	luminositySlider = new MySlider(Qt::Horizontal);
	luminositySlider->setEnabled(false);
	QObject::connect(luminositySlider,SIGNAL(valueChanged(int)),this,SLOT(luminosityValueChangedLabelSlot(int)));
	QObject::connect(luminositySlider,SIGNAL(valueModified(int)),this,SLOT(luminosityValueChangedSlot(int)));

	layoutLuminosity->addWidget(luminositySlider);
	layoutLuminosity->addWidget(luminosityValue);
	layoutLuminosity->setAlignment(luminosityValue,Qt::AlignHCenter);
	luminositySlider->setFixedHeight(20);
	luminosityBox->setFixedHeight(70);
	luminositySlider->setMaximum(100);
	luminositySlider->setMinimum(-100);

	//Reset button
	resetButton = new QPushButton("Reset");
	resetButton->setEnabled(false);
	QObject::connect(resetButton,SIGNAL(clicked()),this,SLOT(resetSlot()));
	


	layoutPrincipal->setAlignment(Qt::AlignTop);
	layoutPrincipal->addWidget(contrastBox);
	layoutPrincipal->addWidget(luminosityBox);
	layoutPrincipal->addWidget(resetButton);
	tabWidget->addTab(centralWidget,"Adjustements");


	////////////RGB//////////
	QWidget *centralWidgetRGB = new QWidget();
	QFormLayout *layoutRGB = new QFormLayout();

	m_red = new QLineEdit();
	m_green = new QLineEdit();
	m_blue = new QLineEdit();
	QPushButton *createImage = new QPushButton("Create Image");
	layoutRGB->addRow("Red band",m_red);
	layoutRGB->addRow("Green band",m_green);
	layoutRGB->addRow("Blue band",m_blue);
	layoutRGB->addRow(createImage);

	centralWidgetRGB->setLayout(layoutRGB);
	tabWidget->addTab(centralWidgetRGB,"RGB");

	QObject::connect(createImage,SIGNAL(clicked()),this,SLOT(createImageSlot()));

	//////Textures///////
	QScrollArea *scrollAreaTexture = new QScrollArea();
	QWidget *centralWidgetTexture = new QWidget();	
	QVBoxLayout *layoutTexture = new QVBoxLayout();

	QGroupBox *groupBox = new QGroupBox();
	QHBoxLayout *choiceLayout = new QHBoxLayout();
	m_lbpChoice = new QRadioButton("LBP");
	m_lbpChoice->setChecked(true);
	QObject::connect(m_lbpChoice,SIGNAL(toggled(bool)),this,SLOT(lbpExpand(bool)));
	m_mgrfChoice = new QRadioButton("GMRF");
	QObject::connect(m_mgrfChoice,SIGNAL(toggled(bool)),this,SLOT(gmrfExpand(bool)));
	choiceLayout->addWidget(m_lbpChoice);
	choiceLayout->addWidget(m_mgrfChoice);
	groupBox->setLayout(choiceLayout);
	groupBox->setFixedHeight(35);
	groupBox->setAlignment(Qt::AlignTop);
	layoutTexture->setAlignment(Qt::AlignTop);
	layoutTexture->addWidget(groupBox);

	//Texture Method
	m_selectionButton = new QPushButton("Patern selection");
	QObject::connect(m_selectionButton,SIGNAL(toggled(bool)),this,SLOT(selectionPatern(bool)));
	m_selectionButton->setCheckable(true);
	layoutTexture->addWidget(m_selectionButton);



	m_table = new QTableWidget();
	m_table->setMaximumHeight(170);
	m_table->setMaximumWidth(200);
	m_table->setColumnCount(3);
	m_table->setColumnWidth(0,85);
	m_table->setColumnWidth(1,50);
	m_table->setColumnWidth(2,50);
	m_table->setFixedHeight(170);
	QStringList list = QString("Index Mean,Color,Visible ").split(",");
	m_table->setHorizontalHeaderLabels(list);
	layoutTexture->addWidget(m_table);
	
	
	//Display Patern
	m_patternView = new QGraphicsView();
	m_patternView->setMaximumHeight(170);
	m_patternView->setMaximumWidth(200);
	layoutTexture->addWidget(m_patternView);

	//Display Histogram
	m_plot = new MyQWTPlot();
	layoutTexture->addWidget(m_plot);

	

	//Selection of research zone
	QWidget *selectionWidget = new QWidget();
	QHBoxLayout *selectionLayout = new QHBoxLayout();
	m_selectionZoneButton = new QPushButton("Research Zone");
	m_allImageZone = new QPushButton("All image");
	QObject::connect(m_selectionZoneButton,SIGNAL(toggled(bool)),this,SLOT(selectionZone(bool)));
	m_selectionZoneButton->setCheckable(true);
	selectionLayout->addWidget(m_selectionZoneButton);
	selectionLayout->addWidget(m_allImageZone);
	selectionWidget->setLayout(selectionLayout);
	layoutTexture->addWidget(selectionWidget);
	m_selectionZoneButton->setEnabled(false);
	m_allImageZone->setEnabled(false);
	QObject::connect(m_allImageZone,SIGNAL(clicked()),this,SLOT(allImageSlot()));

	//Reset button
	resetGMRF = new QPushButton("Reset");
	QObject::connect(resetGMRF,SIGNAL(clicked()),this,SLOT(resetGMRFSlot()));
	layoutTexture->addWidget(resetGMRF);
	resetGMRF->setEnabled(false);
	

	//Advanced Options
	QPushButton *advancedSettingsButton = new QPushButton("Advanced Settings");
	advancedSettingsButton->setCheckable(true);
	QObject::connect(advancedSettingsButton,SIGNAL(toggled(bool)),this,SLOT(expandAdvancedSettings(bool)));
	m_advancedSettingsGroup = new QGroupBox();
	m_advancedSettingsGroup->setVisible(false);
	QVBoxLayout *advancedSettingsLayout = new QVBoxLayout();
	QGroupBox *colorizeOptions = new QGroupBox("Colors");
	QHBoxLayout *colorizeOptionsLayout = new QHBoxLayout();
	m_opaqueChoice = new QRadioButton("Opaque");
	m_nonOpaqueChoice = new QRadioButton("Non Opaque");
	m_nonOpaqueChoice->setChecked(true);
	colorizeOptionsLayout->addWidget(m_opaqueChoice);
	colorizeOptionsLayout->addWidget(m_nonOpaqueChoice);
	colorizeOptions->setLayout(colorizeOptionsLayout);
	advancedSettingsLayout->addWidget(colorizeOptions);
	m_advancedSettingsGroup->setLayout(advancedSettingsLayout);
	layoutTexture->addWidget(advancedSettingsButton);
	layoutTexture->addWidget(m_advancedSettingsGroup);

	QGroupBox *neighborsGroup = new QGroupBox("Neighbors");
	QHBoxLayout *neighborsLayout = new QHBoxLayout();
	m_neighborsList = new QComboBox();
	QObject::connect(m_neighborsList,SIGNAL(currentIndexChanged(int)),this,SLOT(neighborsChangeSlot(int)));
	m_neighborsList->addItems(QString("3x3,5x5,7x7,9x9").split(","));
	neighborsLayout->addWidget(m_neighborsList);
	neighborsGroup->setLayout(neighborsLayout);
	advancedSettingsLayout->addWidget(neighborsGroup);
	
	centralWidgetTexture->setLayout(layoutTexture);
	scrollAreaTexture->setWidget(centralWidgetTexture);
	tabWidget->addTab(scrollAreaTexture,"Texture");

}
void ToolsDock::resizeEvent(QResizeEvent *event){
	emit resizeTools(width());
}
void ToolsDock::contrastValueChangedSlot(int value){
	if (!m_automaticChange){
		emit contrastLuminosityModified(contrastSlider->value(),luminositySlider->value());
	}
	else m_automaticChange =false;

}
void ToolsDock::luminosityValueChangedSlot(int value){
	if (!m_automaticChange){
		emit contrastLuminosityModified(contrastSlider->value(),luminositySlider->value());
	}
	else m_automaticChange =false;
}
void ToolsDock::enableSliders(){
	contrastSlider->setEnabled(true);
	luminositySlider->setEnabled(true);
	contrastSlider->setValue(0);
	luminositySlider->setValue(0);
	resetButton->setEnabled(true);
}
void ToolsDock::disableSliders(){
	contrastSlider->setEnabled(false);
	luminositySlider->setEnabled(false);
	resetButton->setEnabled(false);
}
void ToolsDock::resetSlot(){
	contrastSlider->setValue(0);
	luminositySlider->setValue(0);
	emit contrastLuminosityModified(contrastSlider->value(),luminositySlider->value());
}
void ToolsDock::setValues(int contrast,int luminosity){
	m_automaticChange = true;
	luminosityValueChangedSlot(0);
	contrastSlider->setValue(contrast);
	luminositySlider->setValue(luminosity);
}
void ToolsDock::createImageSlot(){
	if (!m_red->text().isEmpty() && !m_green->text().isEmpty() && !m_blue->text().isEmpty()){
		emit createRGBImage(m_red->text(),m_green->text(),m_blue->text());
	}
	else {
		QMessageBox::warning(this, "Warning", "You have to complete all the fields.");
	}
}
void ToolsDock::selectionPatern(bool checked){
	m_selectionZoneButton->setChecked(false);
	emit selectionPaternButton(checked);
}
void ToolsDock::selectionZone(bool checked){
	m_selectionButton->setChecked(false);
	emit selectionZoneButton(checked);
}
void ToolsDock::desactivateSelectionButton(){
	m_selectionButton->setChecked(false);
}
void ToolsDock::desactivateSelectionZoneButton(){
	m_selectionZoneButton->setChecked(false);

}
void ToolsDock::updateLineEdits(double * features){
	double mean = features[0];
	double variance = features[1];
	m_coorInit->setText(QString::number(mean,'g',6));
	m_coorEnd->setText(QString::number(variance,'g',6));
}
void ToolsDock::displayPaternImage(QImage image){
	QGraphicsScene *patternScene = new QGraphicsScene();
	m_patternView->setScene(patternScene);
	patternScene->addPixmap(QPixmap::fromImage(image));
	m_patternView->fitInView(patternScene->sceneRect(), Qt::KeepAspectRatio );
}
bool ToolsDock::getCheckedPaternButton(){
	return m_selectionButton->isChecked();

}
bool ToolsDock::getCheckedZoneButton(){
	return m_selectionZoneButton->isChecked();

}
void ToolsDock::enableLBPTab(bool enabled){
	tabWidget->setTabEnabled(2,enabled);
}
void ToolsDock::addLabel(){
	m_table->setRowCount(m_table->rowCount()+1);
	QTableWidgetItem *item = new QTableWidgetItem();
	item->setBackgroundColor(Qt::GlobalColor(colors[m_table->rowCount()-1]));
	m_table->setItem(m_table->rowCount()-1,2,item);
	if (m_table->rowCount()>0){
		enableResetButtonMGRF(true);
		m_allImageZone->setEnabled(true);
	}
	else {
		enableResetButtonMGRF(false);
		m_allImageZone->setEnabled(false);
	}
}
void ToolsDock::lbpAddLabel(QString vector){
	QVector<int> colors;
	colors.push_back(7);
	colors.push_back(8);
	colors.push_back(9);
	colors.push_back(11);
	colors.push_back(12);
	colors.push_back(10);
	colors.push_back(13);
	colors.push_back(14);
	colors.push_back(15);
	m_table->setRowCount(m_table->rowCount()+1);
	m_table->setItem(m_table->rowCount()-1,0,new QTableWidgetItem(vector));
	QTableWidgetItem *item = new QTableWidgetItem();
	item->setBackgroundColor(Qt::GlobalColor(colors[m_table->rowCount()-1]));
	m_table->setItem(m_table->rowCount()-1,1,item);
	CheckBoxTable *checkBox = new CheckBoxTable(m_table->rowCount()-1);
	m_table->setCellWidget(m_table->rowCount()-1,2,checkBox);
	QObject::connect(checkBox,SIGNAL(checkBoxToggled(bool,int)),this,SLOT(setCurveVisible(bool,int)));
	if (m_table->rowCount()>0){
		enableResetButtonMGRF(true);
		m_allImageZone->setEnabled(true);
	}
	else {
		enableResetButtonMGRF(false);
		m_allImageZone->setEnabled(false);
	}
}
void ToolsDock::resetGMRFSlot(){
	m_table->setRowCount(0);
	m_plot->reset();
	emit resetGMRFSignal();

}
void ToolsDock::enableSearchingZoneButton(bool enable){
	m_selectionZoneButton->setEnabled(enable);

}
void ToolsDock::enableResetButtonMGRF(bool enable){
	resetGMRF->setEnabled(enable);
}
void ToolsDock::contrastValueChangedLabelSlot(int value){
	contrastValue->setText(QString::number(value));
}
void ToolsDock::luminosityValueChangedLabelSlot(int value){
	luminosityValue->setText(QString::number(value));
}
void ToolsDock::lbpExpand(bool toggled){
	if(toggled){
		m_table->setRowCount(0);
		m_table->setColumnCount(2);
		m_table->setColumnWidth(0,85);
		m_table->setColumnWidth(1,85);
		m_table->setFixedHeight(170);
		QStringList list = QString("Index Mean,Color,Visible ").split(",");
		m_table->setHorizontalHeaderLabels(list);
	}
}
void ToolsDock::gmrfExpand(bool toggled){
	if(toggled) {	
		m_table->setRowCount(0);
		m_table->setColumnCount(3);
		m_table->setColumnWidth(0,85);
		m_table->setColumnWidth(1,85);
		m_table->setColumnWidth(2,25);
		m_table->setFixedHeight(170);
		QStringList list = QString("Mean,Variance, ").split(",");
		m_table->setHorizontalHeaderLabels(list);
	}
}
void ToolsDock::allImageSlot(){
	emit allImageZoneSignal();
}
int ToolsDock::getTextureMethod(){
	if(m_lbpChoice->isChecked()){
		return 0; 
	}
	else if (m_mgrfChoice->isChecked()){
		return 1;
	}
	else return -1;
}
void ToolsDock::expandAdvancedSettings(bool toggled){
	if (toggled) m_advancedSettingsGroup->setVisible(true);
	else m_advancedSettingsGroup->setVisible(false);
}
bool ToolsDock::getColorsChoice(){
	if(m_opaqueChoice->isChecked()) return 0;
	else return 1;
}
int ToolsDock::getNeighBorsChoice(){
	return m_neighborsList->currentIndex();
}
void ToolsDock::neighborsChangeSlot(int index){
	emit neighborsChangeSignal();
}
void ToolsDock::resetTable(){
	m_table->setRowCount(0);
}
void ToolsDock::addHistogramPublic(QVector<double> histogram,int color,int neighborhood){
	m_plot->addHistogram(histogram,color,neighborhood);
}
void ToolsDock::addGaussianPublic(double* data,int color){
	m_plot->addGaussian(data,color);
}
int ToolsDock::getColorCurrentLabel(){
	return colors[m_table->rowCount()-1];
}
void ToolsDock::setColorsSet(){
	colors.push_back(7);
	colors.push_back(8);
	colors.push_back(9);
	colors.push_back(11);
	colors.push_back(12);
	colors.push_back(10);
	colors.push_back(13);
	colors.push_back(14);
	colors.push_back(15);
}
void ToolsDock::setCurveVisible(bool checked,int labelNumber){
		m_plot->setVisibleQwtCurve(checked, labelNumber);
}
