#ifndef DEF_TOOLSDOCK
#define DEF_TOOLSDOCK

#include <QtWidgets>
#include "MySlider.h"
#include <C:\Qwt-6.1.2\include\qwt_plot.h>
#include <C:\Qwt-6.1.2\include\qwt_plot_curve.h>
#include <C:\Qwt-6.1.2\include\qwt_plot_grid.h>
#include <C:\Qwt-6.1.2\include\qwt_symbol.h>
#include <C:\Qwt-6.1.2\include\qwt_legend.h>
#include "MyQWTPlot.h"
#include "CheckBoxTable.h"


class ToolsDock : public QDockWidget{
	Q_OBJECT
public:
	ToolsDock(QString title);
	void resizeEvent(QResizeEvent *event);
	void enableSliders();
	void disableSliders();
	void setValues(int contrast,int luminosity);
	void desactivateSelectionButton();
	void desactivateSelectionZoneButton();
	void updateLineEdits(double* features);
	void displayPaternImage(QImage image);
	bool getCheckedPaternButton();
	bool getCheckedZoneButton();
	void enableLBPTab(bool enabled);
	void addLabel();
	void enableSearchingZoneButton(bool enable);
	void enableResetButtonMGRF(bool enable);
	int getTextureMethod();
	void lbpAddLabel(QString vector);
	bool getColorsChoice();
	int getNeighBorsChoice();
	void resetTable();
	void addHistogramPublic(QVector<double> histogram, int color,int neighborhood);
	void addGaussianPublic(double* data, int color);
	int getColorCurrentLabel();

protected:
	void setColorsSet();


public slots:
	void contrastValueChangedSlot(int value);
	void luminosityValueChangedSlot(int value);
	void resetSlot();
	void createImageSlot();
	void selectionPatern(bool checked);
	void selectionZone(bool checked);
	void resetGMRFSlot();
	void contrastValueChangedLabelSlot(int value);
	void luminosityValueChangedLabelSlot(int value);
	void lbpExpand(bool toggled);
	void gmrfExpand(bool toggled);
	void allImageSlot();
	void expandAdvancedSettings(bool toggled);
	void neighborsChangeSlot(int index);
	void setCurveVisible(bool checked,int labelNumber);

signals:
	void resizeTools(int widthTools);
	void contrastLuminosityModified(int contrast,int luminosity);
	void createRGBImage(QString redBand,QString greenBand,QString blueBand);
	void selectionPaternButton(bool checked);
	void selectionZoneButton(bool checked);
	void resetGMRFSignal();
	void allImageZoneSignal();
	void neighborsChangeSignal();

private:
	MySlider *luminositySlider;
	MySlider *contrastSlider;
	QLabel *contrastValue;
	QLabel *luminosityValue;
	QPushButton *resetButton;
	QLineEdit *m_red;
	QLineEdit *m_green;
	QLineEdit *m_blue;
	QPushButton *m_selectionButton;
	QLineEdit *m_coorInit;
	QLineEdit *m_coorEnd;
	QGraphicsView *m_patternView;
	QPushButton *m_selectionZoneButton;
	QTabWidget *tabWidget;
	QTableWidget *m_table ;
	QPushButton *resetGMRF;
	bool m_automaticChange;
	QPushButton *m_allImageZone;
	QRadioButton *m_lbpChoice;
	QRadioButton *m_mgrfChoice;
	QGroupBox *m_advancedSettingsGroup;
	QRadioButton *m_opaqueChoice;
	QRadioButton *m_nonOpaqueChoice;
	QComboBox *m_neighborsList;
	MyQWTPlot *m_plot;
	QVector<int> colors;

};





#endif